/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.report.AddReportsViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.rational.report.web.core.app.internal.common.SelectionWidget");
dojo.require("com.ibm.rational.report.web.core.app.view.report.AddReportsDialog");
dojo.require("com.ibm.rational.report.web.core.client.internal.ReportClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");

var ServiceabilityMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var ReportMessages			= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.report", "Messages");
var ReportClient 			= com.ibm.rational.report.web.core.client.internal.ReportClient;
var session					= com.ibm.team.repository.web.client.session;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

/**
 * Displays the Add Report to Folder view.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.report.AddReportsViewer",
	[dijit._Widget, dijit._Templated],	{
	
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/view/report/templates/AddReportsViewer.html"),
	
	constructor: function(args) {
		this._args = args;
	},
	
	postCreate: function() {
		this._controller = this._args.controller;
		this._selectedNodeId = this._args.nodeId;
		this._selectionWidget = null;
		
		var self = this;
		
		var addReportsDialog = new com.ibm.rational.report.web.core.app.view.report.AddReportsDialog(
			{controller: this._controller, title: ReportMessages.ADD_REPORTS_VIEWER_PAGE_TITLE,
			 nodeId: this._args.nodeId, postSave: this._args.postSave, postCancel: this._args.postCancel});	
				
		dojo.connect(addReportsDialog.searchButton, "onclick", addReportsDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				var projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
				var owner = session.getAuthenticatedContributor();
				var srh = new ServiceResponseHandler(self, "_getAllReports", "_getAllReportsError");
				ReportClient.getAllReports(srh, {projectAreaId: projectArea.itemId, ownerId: owner.itemId,
					pattern: addReportsDialog.searchReports.value});
			}
		);
		
		dojo.connect(addReportsDialog.okButton, "onclick", addReportsDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				self._handleSave();
				addReportsDialog.close();
			}
		);
		
		dojo.connect(addReportsDialog.cancelButton, "onclick", addReportsDialog.dialog,
			function (e) {
				dojo.stopEvent(e);
				addReportsDialog.close();
			}
		);
		
		this._addReportsDialog = addReportsDialog;
		this._populateSelectionWidget();
	},
	
	_getAllReports: function(reports) {
		this._allReports = reports;
		this._populateSelectionWidget();
	},
	
	_getAllReportsError: function() {
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_RETRIEVE_REPORT_DESIGNS, error);
	},
	
	_populateSelectionWidget: function() {
		var projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
		var owner = session.getAuthenticatedContributor();		
		var params = {projectAreaId: projectArea.itemId, ownerId: owner.itemId, parentId: this._selectedNodeId, reportNodesOnly: true};
		var srh = new ServiceResponseHandler(this, "_fetchChildrenSuccess", "_fetchChildrenError");				
		ReportClient.getReportTreeNodes(srh, params);
	},
	
	_fetchChildrenSuccess: function(nodes) {
		if (nodes == null) {
			nodes = [];
		}	
		var self = this;
		var availableOptions = [];
		var selectedOptions = [];

		var reports = this._allReports;
		
		if (reports == null) {
			reports = [];
		}
		
		for (var i = 0; i < reports.length; i++) {
			var present = false;
			for (var j = 0; j < nodes.length; j++) {
			   	if (nodes[j].report) {
					if (nodes[j].report.itemId == reports[i].itemId) {
						present = true;
						break;
					}
			   	}
			}
			if (!present)
				availableOptions.push({value:reports[i].itemId, text:reports[i].name});				
		}	
			
		for (var i = 0; i < nodes.length; i++) {
			selectedOptions.push({value:nodes[i].itemId, text:nodes[i].name});
		}
		
		if (this._selectionWidget != null) {
			this._addReportsDialog.selectionWidgetContainer.removeChild(this._selectionWidget.domNode);
		}
		
		this._selectionWidget = new com.ibm.rational.report.web.core.app.internal.common.SelectionWidget(
			{availableOptions:availableOptions, selectedOptions:selectedOptions,
			 availableItemsTitle: ReportMessages.ADD_REPORTS_VIEWER_AVAILABLE_REPORTS_LABEL,
			 selectedItemsTitle: ReportMessages.ADD_REPORTS_VIEWER_ASSIGNED_REPORTS_LABEL,
			 leftToRightBtnText: ReportMessages.ADD_REPORTS_VIEWER_ASSIGN_BUTTON_LABEL,
			 rightToLeftBtnText: ReportMessages.ADD_REPORTS_VIEWER_REMOVE_BUTTON_LABEL});
		 
		dojo.connect(this._selectionWidget.availableControl, "onchange",
			function(e) {
				e.preventDefault();
				self._addReportsDialog.okButton.disabled = false;
			}
		);
		
		dojo.connect(this._selectionWidget.selectedControl, "onchange",
			function(e) {
				e.preventDefault();
				self._addReportsDialog.okButton.disabled = false;
			}
		);
	
		self._addReportsDialog.selectionWidgetContainer.appendChild(this._selectionWidget.domNode);
	},
	
	_fetchChildrenError: function() {
		//TODO: do an appropriate action
	},

	_handleSave: function() {
		var owner = session.getAuthenticatedContributor();
		
		var removed = this._selectionWidget.getRemoved();
		
		var self = this;
	
		var srh = new ServiceResponseHandler(this, "_createNewReportTreeNodes", "_deleteReportTreeNodesError");	
		var ids = [];	
		for (var i = 0; i < removed.length; i++) {
			if (removed[i] != null) {				
				ids.push(removed[i].value);
			}	
		}
		if (ids.length > 0) {
			ReportClient.postDeleteReportTreeNodes(srh, {
				ids: ids,
				deleteReferencedReports: false
			});
		}
		else {
			this._createNewReportTreeNodes();
		}		
	},
	
	_deleteReportTreeNodesError: function(error) {
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_REMOVE_SELECETED_REPORTS, error);
		this._createNewReportTreeNodes();
	},
	
	_createNewReportTreeNodes: function() {
		var parentFolderId = this._selectedNodeId;

		var owner = session.getAuthenticatedContributor();
		var projectAreaId = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea().itemId;
		
		var added = this._selectionWidget.getAdded();
		
		var reportItemIds = [];
		var srh = new ServiceResponseHandler(this, "_createNewReportTreeNodesSuccess", "_remoteError");		
		for (var i = 0; i < added.length; i++) {
			if (added[i] != null) {
				reportItemIds.push(added[i].value);
			}				
		}
		if (reportItemIds.length > 0) {
			var params = {
				reportItemIds: reportItemIds,
				parentFolderId: parentFolderId,
				projectAreaId: projectAreaId,
				ownerId: owner.itemId
			};
			ReportClient.postNewReportTreeNodes(srh, params);
		}
		else {
			this._createNewReportTreeNodesSuccess();
		}
		
	},
	
	_createNewReportTreeNodesSuccess: function() {
		this._controller.syncReportNodeChanged();
		this._controller.reportTreeParser.expandFolder(this._selectedNodeId);
		if (this._postSave) {
			this._postSave();
		}
		else {
			this._controller.defaultAction();
		}
							
	},
	
	_remoteError: function(error) {
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_ADD_SELECETED_REPORTS, error);
	}
	
});

})();